
package eu.dnetlib.oa.graph.usagestatsbuild.export;

import java.sql.*;
import java.text.SimpleDateFormat;
import java.util.*;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author D. Pierrakos
 */
public class EpisciencesViewsDownloads {

	private String logPath;

	private Statement stmt = null;

	private static final Logger logger = LoggerFactory.getLogger(EpisciencesViewsDownloads.class);

	public void processEpisciencesLogs() throws Exception {
		try {

			logger.info("Views Episciences processing starts at: " + new Timestamp(System.currentTimeMillis()));
			episciencesViewsStats();
			logger.info("Views Episciences processing ends at: " + new Timestamp(System.currentTimeMillis()));

			logger.info("downloads Episciences processing starts at: " + new Timestamp(System.currentTimeMillis()));
			episciencesDownloadsStats();
			logger.info("Downloads Episciences processing ends at: " + new Timestamp(System.currentTimeMillis()));

		} catch (Exception e) {
			logger.error("Failed to process logs: " + e);
			throw new Exception("Failed to process logs: " + e.toString(), e);
		}
	}

	public void episciencesViewsStats() throws Exception {
		logger.info("Creating episciences Views");

		Statement stmt = ConnectDB.getHiveConnection().createStatement();
		ConnectDB.getHiveConnection().setAutoCommit(false);

		logger.info("Dropping Episcience Views Table");
		String dropEpisciencesViewsTable = "DROP TABLE IF EXISTS " + ConnectDB.getUsageStatsDBSchema()
			+ ".episciencesviews ";
		stmt.executeUpdate(dropEpisciencesViewsTable);
		logger.info("Dropped Episcience Views Table");

		logger.info("Creating Episcience Views Table");
		String createEpisciencesViewsTable = "CREATE TABLE IF NOT EXISTS " + ConnectDB.getUsageStatsDBSchema()
			+ ".episciencesviews (source STRING, repository_id STRING, result_id STRING, date STRING, count INT, openaire INT)"
			+ " clustered by (source) into 100 buckets stored as orc tblproperties('transactional'='true') ";

		stmt.executeUpdate(createEpisciencesViewsTable);

		String returnEpisciencesJournals = "SELECT id, substring(regexp_extract(websiteurl,'^([^\\.]+)\\.?',1),9) FROM "
			+ ConnectDB.getStatsDBSchema() +
			".datasource  where websiteurl like '%episciences%' and (dateofvalidation is not null or harvested=true) "
			+ "and substring(regexp_extract(websiteurl,'^([^\\\\.]+)\\\\.?',1),9)!='episciences'";

		PreparedStatement st = ConnectDB.DB_HIVE_CONNECTION
			.prepareStatement(returnEpisciencesJournals);
		ResultSet rs = st.executeQuery();
		while (rs.next()) {
			String journal_openaire_id = rs.getString(1);
			String episciencesSuffix = rs.getString(2);

			logger.info("Working on journal_id:" + journal_openaire_id + " suffix:" + episciencesSuffix);
			logger.info("Dropping episciencesSuffix_result_views_monthly_tmp table");
			String dropepisciencesSuffixView = "DROP VIEW " + ConnectDB.getUsageStatsDBSchema()
				+ "." + episciencesSuffix.replace("-", "_") + "_result_views_monthly_tmp";
			// Statement stmtRS = ConnectDB.getHiveConnection().createStatement();
			stmt.executeUpdate(dropepisciencesSuffixView);
			logger.info("Dropped episciencesSuffix_result_views_monthly_tmp table");

			logger.info("Creating episciencesSuffix_result_views_monthly_tmp table");

			String create_result_views_monthly = "CREATE OR REPLACE VIEW " + ConnectDB.getUsageStatsDBSchema()
				+ "." + episciencesSuffix.replace("-", "_") + "_result_views_monthly_tmp "
				+ "AS SELECT entity_id, "
				+ "reflect('java.net.URLDecoder', 'decode', entity_id) AS id,"
				+ "COUNT(entity_id) as views, SUM(CASE WHEN referrer_name LIKE '%openaire%' THEN 1 ELSE 0 END) "
				+ "AS openaire_referrer, "
				+ "CONCAT(YEAR(timestamp), '/', LPAD(MONTH(timestamp), 2, '0')) AS month, source "
				+ "FROM " + ConnectDB.getUsageStatsDBSchema()
				+ ".episcienceslogdistinct where action='action' and (source_item_type='oaItem' or "
				+ "source_item_type='repItem') and entity_id like '%" + episciencesSuffix + "%'"
				+ "GROUP BY entity_id, CONCAT(YEAR(timestamp), '/', LPAD(MONTH(timestamp), 2, '0')), "
				+ "source ORDER BY source, entity_id";
			stmt.executeUpdate(create_result_views_monthly);
			logger.info("Created episciencesSuffix_result_views_monthly_tmp table");

			logger.info("Inserting episciencesSuffix_result_views_monthly_tmp into EpisciencesViews Table");
			String insertIntoEpisciencesViewsTable = "INSERT INTO " + ConnectDB.getUsageStatsDBSchema()
				+ ".episciencesviews SELECT 'Episciences' as source, '"
				+ journal_openaire_id + "' as repository_id, ro.id as result_id, month as date,"
				+ " max(views) AS count, max(openaire_referrer) AS openaire "
				+ "FROM " + ConnectDB.getUsageStatsDBSchema()
				+ "." + episciencesSuffix.replace("-", "_") + "_result_views_monthly_tmp p,"
				+ ConnectDB.getStatsDBSchema()
				+ ".result_oids ro WHERE p.id=ro.oid  GROUP BY ro.id, month ORDER BY ro.id, month";

			stmt.executeUpdate(insertIntoEpisciencesViewsTable);
			logger.info("Inserted episciencesSuffix_result_views_monthly_tmp into EpisciencesViews Table");

			stmt.executeUpdate(dropepisciencesSuffixView);
			logger.info("Dropped episciencesSuffix_result_views_monthly_tmp view");
		}
		rs.close();

		logger.info("Episciences Views Created");
	}

	public void episciencesDownloadsStats() throws Exception {
		logger.info("Creating episciences Downloads");

		Statement stmt = ConnectDB.getHiveConnection().createStatement();
		ConnectDB.getHiveConnection().setAutoCommit(false);

		logger.info("Dropping Episcience Downloads Table");
		String dropEpisciencesDownloadsTable = "DROP TABLE IF EXISTS " + ConnectDB.getUsageStatsDBSchema()
			+ ".episciencesvdownloads ";
		stmt.executeUpdate(dropEpisciencesDownloadsTable);
		logger.info("Dropped Episcience Downloads Table");

		logger.info("Creating Episcience Downloads Table");
		String createEpisciencesDownloadsTable = "CREATE TABLE IF NOT EXISTS " + ConnectDB.getUsageStatsDBSchema()
			+ ".episciencesdownloads (source STRING, repository_id STRING, result_id STRING, date STRING, count INT, openaire INT)"
			+ " clustered by (source) into 100 buckets stored as orc tblproperties('transactional'='true') ";

		stmt.executeUpdate(createEpisciencesDownloadsTable);

		String returnEpisciencesJournals = "SELECT id, substring(regexp_extract(websiteurl,'^([^\\.]+)\\.?',1),9) FROM "
			+ ConnectDB.getStatsDBSchema() +
			".datasource  where websiteurl like '%episciences%' and (dateofvalidation is not null or harvested=true) "
			+ "and substring(regexp_extract(websiteurl,'^([^\\\\.]+)\\\\.?',1),9)!='episciences'";

		PreparedStatement st = ConnectDB.DB_HIVE_CONNECTION
			.prepareStatement(returnEpisciencesJournals);
		ResultSet rs = st.executeQuery();
		while (rs.next()) {
			String journal_openaire_id = rs.getString(1);
			String episciencesSuffix = rs.getString(2);

			logger.info("Working on journal_id:" + journal_openaire_id + " suffix:" + episciencesSuffix);
			logger.info("Dropping episciencesSuffix_result_downloads_monthly_tmp table");
			String dropepisciencesSuffixDownloads = "DROP VIEW IF EXISTS " + ConnectDB.getUsageStatsDBSchema()
				+ "." + episciencesSuffix.replace("-", "_") + "_result_downloads_monthly_tmp";
			stmt.executeUpdate(dropepisciencesSuffixDownloads);

			logger.info("Creating episciencesSuffix_result_downloads_monthly_tmp table");

			String create_result_downloads_monthly = "CREATE OR REPLACE VIEW " + ConnectDB.getUsageStatsDBSchema()
				+ "." + episciencesSuffix.replace("-", "_") + "_result_downloads_monthly_tmp "
				+ "AS SELECT entity_id, "
				+ "reflect('java.net.URLDecoder', 'decode', entity_id) AS id,"
				+ "COUNT(entity_id) as views, SUM(CASE WHEN referrer_name LIKE '%openaire%' THEN 1 ELSE 0 END) "
				+ "AS openaire_referrer, "
				+ "CONCAT(YEAR(timestamp), '/', LPAD(MONTH(timestamp), 2, '0')) AS month, source "
				+ "FROM " + ConnectDB.getUsageStatsDBSchema()
				+ ".episcienceslogdistinct where action='download' and (source_item_type='oaItem' or "
				+ "source_item_type='repItem') and entity_id like '%" + episciencesSuffix + "%'"
				+ "GROUP BY entity_id, CONCAT(YEAR(timestamp), '/', LPAD(MONTH(timestamp), 2, '0')), "
				+ "source ORDER BY source, entity_id";

			stmt.executeUpdate(create_result_downloads_monthly);
			logger.info("Created episciencesSuffix_result_downloads_monthly_tmp table");

			logger.info("Inserting episciencesSuffix_result_downloads_monthly_tmp into EpisciencesDownloadsTable");
			String insertIntoEpisciencesDownloadsTable = "INSERT INTO " + ConnectDB.getUsageStatsDBSchema()
				+ ".episciencesdownloads SELECT 'Episciences' as source, '"
				+ journal_openaire_id + "' as repository_id, ro.id as result_id, month as date,"
				+ " max(views) AS count, max(openaire_referrer) AS openaire "
				+ "FROM " + ConnectDB.getUsageStatsDBSchema()
				+ "." + episciencesSuffix.replace("-", "_") + "_result_downloads_monthly_tmp p,"
				+ ConnectDB.getStatsDBSchema()
				+ ".result_oids ro WHERE p.id=ro.oid  GROUP BY ro.id, month ORDER BY ro.id, month";

			stmt.executeUpdate(insertIntoEpisciencesDownloadsTable);
			logger.info("Inserted episciencesSuffix_result_downloads_monthly_tmp into EpisciencesDownloadsTable");

			stmt.executeUpdate(dropepisciencesSuffixDownloads);
			logger.info("Dropped episciencesSuffix_result_downloads_monthly_tmp view");

		}
		rs.close();
	}

	private Connection getConnection() throws SQLException {
		return ConnectDB.getHiveConnection();
	}
}
